'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var esUtils = require('@open-tech-world/es-utils');

function getValue(str) {
    if (!str) {
        return true;
    }
    if (new RegExp(/^["'].*["']$/).test(str)) {
        const regExpRes = new RegExp(/^["'](.*)["']$/).exec(str);
        if (regExpRes) {
            return regExpRes[1];
        }
    }
    if (esUtils.isNumber(str)) {
        return Number(str);
    }
    if (new RegExp(/^(?:.+),(?:.+)$/).test(str)) {
        const arr = [];
        const regExp = new RegExp(/([^,]+)/g);
        let match;
        while ((match = regExp.exec(str)) !== null) {
            arr.push(getValue(match[0]));
        }
        return arr;
    }
    if (new RegExp(/^(?:[a-zA-Z0-9.]+)=(?:.*)$/).test(str)) {
        const regExpRes = new RegExp(/^([a-zA-Z0-9.]+)=(.*)$/).exec(str);
        if (regExpRes) {
            return esUtils.setInObj({}, regExpRes[1], getValue(regExpRes[2]));
        }
    }
    return str;
}

function parseLongObjOpt(str) {
    const obj = {};
    const result = new RegExp(/^-{2}([a-zA-Z0-9.]{2,})(?:=(.*))?$/).exec(str);
    if (result) {
        esUtils.setInObj(obj, result[1], getValue(result[2]));
    }
    return obj;
}

function parseLongOpt(str) {
    const obj = {};
    const result = new RegExp(/^-{2}([a-zA-Z0-9-]{2,})(?:=(.*))?$/).exec(str);
    if (result) {
        const opt = esUtils.camelCase(result[1]);
        const value = result[2];
        obj[opt] = getValue(value);
    }
    return obj;
}

function parseNegOpt(str) {
    const obj = {};
    const result = new RegExp(/^-{2}no-([a-zA-Z-]+)$/).exec(str);
    if (result) {
        const opt = result[1];
        const prop = opt.length === 1 ? opt : esUtils.camelCase(opt);
        obj[prop] = false;
    }
    return obj;
}

function parseShortOpts(str) {
    const obj = {};
    const result = new RegExp(/^(?:-{1})([a-zA-Z]+)$/).exec(str);
    if (result) {
        const opts = result[1];
        for (let i = 0; i < opts.length; i++) {
            obj[opts[i]] = true;
        }
    }
    return obj;
}

function parseShortOptVal(str) {
    const obj = {};
    const result = new RegExp(/^(?:-{1})([a-zA-Z]+)=(.*)$/).exec(str);
    if (result) {
        obj[result[1]] = getValue(result[2]);
    }
    return obj;
}

function setOptsVal(optsObj, newOptions) {
    for (const [key, value] of Object.entries(newOptions)) {
        if (key in optsObj) {
            if (typeof value === 'boolean') {
                optsObj[key] = value;
            }
            if (typeof value === 'number') {
                if (Array.isArray(optsObj[key])) {
                    optsObj[key].push(value);
                }
                else {
                    optsObj[key] = [optsObj[key], value];
                }
            }
            if (typeof value === 'string') {
                if (Array.isArray(optsObj[key])) {
                    optsObj[key].push(value);
                }
                else {
                    optsObj[key] = [optsObj[key], value];
                }
            }
            if (Array.isArray(value)) {
                optsObj[key].push(...value);
                continue;
            }
            if (typeof value === 'object') {
                setOptsVal(optsObj[key], value);
            }
        }
        else {
            optsObj[key] = value;
        }
    }
}

function splitArgs(args) {
    const out = [];
    let str = '';
    for (let i = 0; i < args.length; i++) {
        const char = args[i];
        if (char === '"' || char === "'") {
            const regExpResult = new RegExp(/("[^"]*")|('[^']*')/).exec(args.substring(i));
            if (regExpResult) {
                str += regExpResult[0];
                i = i + regExpResult[0].length - 1;
                continue;
            }
        }
        if (char === ' ') {
            out.push(str);
            str = '';
            continue;
        }
        str += char;
    }
    if (str) {
        out.push(str);
    }
    return out;
}

function isShortOpts(str) {
    return new RegExp(/^(?:-{1})(?:[a-zA-Z]+)$/).test(str);
}
function isShortOptVal(str) {
    return new RegExp(/^(?:-{1})(?:[a-zA-Z]+)=(?:.*)$/).test(str);
}
function isLongOpt(str) {
    return new RegExp(/^-{2}[a-zA-Z0-9-]{2,}(?:=.*)?$/).test(str);
}
function isNegationOpt(str) {
    return new RegExp(/^-{2}no-(?:[a-zA-Z-]+)$/).test(str);
}
function isLongObjOpt(str) {
    return new RegExp(/^-{2}[a-zA-Z0-9.]{2,}(?:=.*)?$/).test(str);
}
function parser(args) {
    const curArgs = typeof args === 'string' ? splitArgs(args.trim()) : args;
    const outObj = { args: [], opts: {} };
    for (let i = 0; i < curArgs.length; i++) {
        const str = curArgs[i];
        if (isShortOpts(str)) {
            const opts = parseShortOpts(str);
            setOptsVal(outObj.opts, opts);
            continue;
        }
        if (isShortOptVal(str)) {
            const opts = parseShortOptVal(str);
            setOptsVal(outObj.opts, opts);
            continue;
        }
        if (isNegationOpt(str)) {
            const opts = parseNegOpt(str);
            setOptsVal(outObj.opts, opts);
            continue;
        }
        if (isLongOpt(str)) {
            const opts = parseLongOpt(str);
            setOptsVal(outObj.opts, opts);
            continue;
        }
        if (isLongObjOpt(str)) {
            const opts = parseLongObjOpt(str);
            setOptsVal(outObj.opts, opts);
            continue;
        }
        outObj.args.push(getValue(str));
    }
    return outObj;
}

exports.parser = parser;
